/* -*- mode: vala; c-basic-offset: 2; tab-width: 8; -*-
 * Application.vala - Application-based window management for AllTray
 * Copyright (c) 2009 Michael B. Trausch <mike@trausch.us>
 * License: GNU GPL v3.0 as published by the Free Software Foundation
 */

#include <glib.h>
#include <glib-object.h>
#include <libwnck/libwnck.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <unistd.h>
#include <sys/types.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <config.h>


#define ALL_TRAY_TYPE_APPLICATION (all_tray_application_get_type ())
#define ALL_TRAY_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ALL_TRAY_TYPE_APPLICATION, AllTrayApplication))
#define ALL_TRAY_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ALL_TRAY_TYPE_APPLICATION, AllTrayApplicationClass))
#define ALL_TRAY_IS_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ALL_TRAY_TYPE_APPLICATION))
#define ALL_TRAY_IS_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ALL_TRAY_TYPE_APPLICATION))
#define ALL_TRAY_APPLICATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ALL_TRAY_TYPE_APPLICATION, AllTrayApplicationClass))

typedef struct _AllTrayApplication AllTrayApplication;
typedef struct _AllTrayApplicationClass AllTrayApplicationClass;
typedef struct _AllTrayApplicationPrivate AllTrayApplicationPrivate;

#define ALL_TRAY_TYPE_PROCESS (all_tray_process_get_type ())
#define ALL_TRAY_PROCESS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ALL_TRAY_TYPE_PROCESS, AllTrayProcess))
#define ALL_TRAY_PROCESS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ALL_TRAY_TYPE_PROCESS, AllTrayProcessClass))
#define ALL_TRAY_IS_PROCESS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ALL_TRAY_TYPE_PROCESS))
#define ALL_TRAY_IS_PROCESS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ALL_TRAY_TYPE_PROCESS))
#define ALL_TRAY_PROCESS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ALL_TRAY_TYPE_PROCESS, AllTrayProcessClass))

typedef struct _AllTrayProcess AllTrayProcess;
typedef struct _AllTrayProcessClass AllTrayProcessClass;

#define ALL_TRAY_DEBUG_TYPE_SUBSYSTEM (all_tray_debug_subsystem_get_type ())

#define ALL_TRAY_DEBUG_TYPE_LEVEL (all_tray_debug_level_get_type ())

typedef enum  {
	ALL_TRAY_APPLICATION_ERROR_FAILED
} AllTrayApplicationError;
#define ALL_TRAY_APPLICATION_ERROR all_tray_application_error_quark ()
struct _AllTrayApplication {
	GObject parent_instance;
	AllTrayApplicationPrivate * priv;
};

struct _AllTrayApplicationClass {
	GObjectClass parent_class;
};

struct _AllTrayApplicationPrivate {
	GList* _windows;
	WnckApplication* _wnckApp;
	GtkStatusIcon* _appIcon;
	AllTrayProcess* _process;
	gboolean _appVisible;
	gboolean _caughtWindow;
	gboolean _usingWindowIcon;
	GQueue* _window_enforce_minimize_queue;
};

typedef enum  {
	ALL_TRAY_DEBUG_SUBSYSTEM_None = 0x00,
	ALL_TRAY_DEBUG_SUBSYSTEM_CommandLine = 0x01,
	ALL_TRAY_DEBUG_SUBSYSTEM_Display = 0x02,
	ALL_TRAY_DEBUG_SUBSYSTEM_TrayIcon = 0x04,
	ALL_TRAY_DEBUG_SUBSYSTEM_WindowManager = 0x08,
	ALL_TRAY_DEBUG_SUBSYSTEM_Process = 0x10,
	ALL_TRAY_DEBUG_SUBSYSTEM_Misc = 0x20,
	ALL_TRAY_DEBUG_SUBSYSTEM_Bug = 0x40,
	ALL_TRAY_DEBUG_SUBSYSTEM_Application = 0x80,
	ALL_TRAY_DEBUG_SUBSYSTEM_AttachHelper = 0x100,
	ALL_TRAY_DEBUG_SUBSYSTEM_Signal = 0x200,
	ALL_TRAY_DEBUG_SUBSYSTEM_Main = 0x400,
	ALL_TRAY_DEBUG_SUBSYSTEM_All = (((((((((ALL_TRAY_DEBUG_SUBSYSTEM_CommandLine | ALL_TRAY_DEBUG_SUBSYSTEM_Display) | ALL_TRAY_DEBUG_SUBSYSTEM_TrayIcon) | ALL_TRAY_DEBUG_SUBSYSTEM_WindowManager) | ALL_TRAY_DEBUG_SUBSYSTEM_Process) | ALL_TRAY_DEBUG_SUBSYSTEM_Misc) | ALL_TRAY_DEBUG_SUBSYSTEM_Bug) | ALL_TRAY_DEBUG_SUBSYSTEM_Application) | ALL_TRAY_DEBUG_SUBSYSTEM_AttachHelper) | ALL_TRAY_DEBUG_SUBSYSTEM_Signal) | ALL_TRAY_DEBUG_SUBSYSTEM_Main
} AllTrayDebugSubsystem;

typedef enum  {
	ALL_TRAY_DEBUG_LEVEL_Information,
	ALL_TRAY_DEBUG_LEVEL_Warning,
	ALL_TRAY_DEBUG_LEVEL_Error,
	ALL_TRAY_DEBUG_LEVEL_Fatal
} AllTrayDebugLevel;

typedef enum  {
	ALL_TRAY_PROCESS_ERROR_SPAWN_FAILED,
	ALL_TRAY_PROCESS_ERROR_PROCESS_DOES_NOT_EXIST,
	ALL_TRAY_PROCESS_ERROR_FAILED
} AllTrayProcessError;
#define ALL_TRAY_PROCESS_ERROR all_tray_process_error_quark ()

extern WnckScreen* all_tray_program_WnckScreen;
extern GList* all_tray_program_WnckEarlyApps;
extern GPid all_tray_program_pgid;
extern gboolean all_tray_program__initially_hide;
static gpointer all_tray_application_parent_class = NULL;

GQuark all_tray_application_error_quark (void);
GType all_tray_application_get_type (void);
GType all_tray_process_get_type (void);
#define ALL_TRAY_APPLICATION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ALL_TRAY_TYPE_APPLICATION, AllTrayApplicationPrivate))
enum  {
	ALL_TRAY_APPLICATION_DUMMY_PROPERTY,
	ALL_TRAY_APPLICATION_WNCK_APP,
	ALL_TRAY_APPLICATION_VISIBLE,
	ALL_TRAY_APPLICATION_CAUGHT_WINDOW
};
GType all_tray_debug_subsystem_get_type (void);
GType all_tray_debug_level_get_type (void);
void all_tray_debug_notification_emit (AllTrayDebugSubsystem subsys, AllTrayDebugLevel lvl, const char* msg);
static void all_tray_application_maybe_setup (AllTrayApplication* self, WnckScreen* scr, WnckApplication* app);
static void _all_tray_application_maybe_setup_wnck_screen_application_opened (WnckScreen* _sender, WnckApplication* app, gpointer self);
static void all_tray_application_bye_wnck_app (AllTrayApplication* self, WnckScreen* scr, WnckApplication* app);
static void _all_tray_application_bye_wnck_app_wnck_screen_application_closed (WnckScreen* _sender, WnckApplication* app, gpointer self);
AllTrayApplication* all_tray_application_new (AllTrayProcess* p);
AllTrayApplication* all_tray_application_construct (GType object_type, AllTrayProcess* p);
static void all_tray_application_maybe_update_window_count (AllTrayApplication* self, WnckScreen* scr, WnckWindow* win);
static void _all_tray_application_maybe_update_window_count_wnck_screen_window_opened (WnckScreen* _sender, WnckWindow* window, gpointer self);
static void _all_tray_application_maybe_update_window_count_wnck_screen_window_closed (WnckScreen* _sender, WnckWindow* window, gpointer self);
GPid all_tray_process_get_pid (AllTrayProcess* self);
GQuark all_tray_process_error_quark (void);
void all_tray_process_run_kinda_fake (AllTrayProcess* self, gint pid, GError** error);
static gboolean all_tray_application_are_we_interested (AllTrayApplication* self, gint pid);
static void all_tray_application_do_setup (AllTrayApplication* self, WnckScreen* scr, WnckApplication* app, gint pid);
void all_tray_application_maybe_setup_for_pid (AllTrayApplication* self, WnckApplication* app, gint pid);
gboolean all_tray_process_get_timer_running (AllTrayProcess* self);
guint all_tray_process_get_timer_event_source (AllTrayProcess* self);
static void all_tray_application_create_icon (AllTrayApplication* self);
static void all_tray_application_display_menu (AllTrayApplication* self, guint button, guint activate_time);
static void _all_tray_application_display_menu_gtk_status_icon_popup_menu (GtkStatusIcon* _sender, guint button, guint activate_time, gpointer self);
static void all_tray_application_update_icon_win_image (AllTrayApplication* self, WnckWindow* win);
static void all_tray_application_update_icon_app_image (AllTrayApplication* self, WnckApplication* app);
static void all_tray_application_hide_all_windows (AllTrayApplication* self);
static GtkMenu* all_tray_application_create_window_list_menu (AllTrayApplication* self);
static void all_tray_application_on_menu_undock (AllTrayApplication* self);
static void _all_tray_application_on_menu_undock_gtk_menu_item_activate (GtkMenuItem* _sender, gpointer self);
static void all_tray_application_on_menu_about (AllTrayApplication* self);
static void _all_tray_application_on_menu_about_gtk_menu_item_activate (GtkMenuItem* _sender, gpointer self);
static void all_tray_application_on_menu_toggle_app (AllTrayApplication* self, GtkMenuItem* item);
static void _all_tray_application_on_menu_toggle_app_gtk_menu_item_activate (GtkMenuItem* _sender, gpointer self);
static void all_tray_application_toggle_window_visibility (AllTrayApplication* self, WnckWindow* w);
static void _lambda0_ (GtkMenuItem* item, AllTrayApplication* self);
static void __lambda0__gtk_menu_item_activate (GtkMenuItem* _sender, gpointer self);
static void _all_tray_application_update_icon_win_image_wnck_window_icon_changed (WnckWindow* _sender, gpointer self);
static void _all_tray_application_update_icon_app_image_wnck_application_icon_changed (WnckApplication* _sender, gpointer self);
static void all_tray_application_on_icon_click (AllTrayApplication* self, GtkStatusIcon* icon);
static void _all_tray_application_on_icon_click_gtk_status_icon_activate (GtkStatusIcon* _sender, gpointer self);
static void all_tray_application_update_icon_name (AllTrayApplication* self, WnckApplication* app);
static void _all_tray_application_update_icon_name_wnck_application_name_changed (WnckApplication* _sender, gpointer self);
static void all_tray_application_toggle_visibility (AllTrayApplication* self);
static char* all_tray_application_get_win_state (AllTrayApplication* self, WnckWindowState state);
static void all_tray_application_debug_msg (AllTrayApplication* self, const char* str);
static void all_tray_application_maintain_hiddenness (AllTrayApplication* self, WnckWindow* win, WnckWindowState changed_bits, WnckWindowState new_state);
static void all_tray_application_set_visibility_for_window (AllTrayApplication* self, WnckWindow* w, gboolean set_visible);
static void all_tray_application_show_all_windows (AllTrayApplication* self);
static void _all_tray_application_maintain_hiddenness_wnck_window_state_changed (WnckWindow* _sender, WnckWindowState changed_mask, WnckWindowState new_state, gpointer self);
static gboolean all_tray_application_set_maintain_hiddenness (AllTrayApplication* self);
static gboolean _all_tray_application_set_maintain_hiddenness_gsource_func (gpointer self);
static void all_tray_application_dialog_destroy (AllTrayApplication* self, GtkAboutDialog* which, gint resp_id);
static void _all_tray_application_dialog_destroy_gtk_dialog_response (GtkAboutDialog* _sender, gint response_id, gpointer self);
WnckApplication* all_tray_application_get_wnck_app (AllTrayApplication* self);
gboolean all_tray_application_get_visible (AllTrayApplication* self);
void all_tray_application_set_visible (AllTrayApplication* self, gboolean value);
gboolean all_tray_application_get_caught_window (AllTrayApplication* self);
void all_tray_application_set_caught_window (AllTrayApplication* self, gboolean value);
static void all_tray_application_finalize (GObject* obj);
static void all_tray_application_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void all_tray_application_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);



GQuark all_tray_application_error_quark (void) {
	return g_quark_from_static_string ("all_tray_application_error-quark");
}


static void _all_tray_application_maybe_setup_wnck_screen_application_opened (WnckScreen* _sender, WnckApplication* app, gpointer self) {
	all_tray_application_maybe_setup (self, _sender, app);
}


static void _all_tray_application_bye_wnck_app_wnck_screen_application_closed (WnckScreen* _sender, WnckApplication* app, gpointer self) {
	all_tray_application_bye_wnck_app (self, _sender, app);
}


AllTrayApplication* all_tray_application_construct (GType object_type, AllTrayProcess* p) {
	AllTrayApplication * self;
	AllTrayProcess* _tmp1_;
	AllTrayProcess* _tmp0_;
	GQueue* _tmp2_;
	g_return_val_if_fail (p != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_process = (_tmp1_ = (_tmp0_ = p, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_process == NULL) ? NULL : (self->priv->_process = (g_object_unref (self->priv->_process), NULL)), _tmp1_);
	_tmp2_ = NULL;
	self->priv->_window_enforce_minimize_queue = (_tmp2_ = g_queue_new (), (self->priv->_window_enforce_minimize_queue == NULL) ? NULL : (self->priv->_window_enforce_minimize_queue = (g_queue_free (self->priv->_window_enforce_minimize_queue), NULL)), _tmp2_);
	all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Application, ALL_TRAY_DEBUG_LEVEL_Information, _ ("Creating a new Application"));
	g_signal_connect_object (all_tray_program_WnckScreen, "application-opened", (GCallback) _all_tray_application_maybe_setup_wnck_screen_application_opened, self, 0);
	g_signal_connect_object (all_tray_program_WnckScreen, "application-closed", (GCallback) _all_tray_application_bye_wnck_app_wnck_screen_application_closed, self, 0);
	if (all_tray_program_WnckEarlyApps != NULL) {
		{
			GList* app_collection;
			GList* app_it;
			app_collection = all_tray_program_WnckEarlyApps;
			for (app_it = app_collection; app_it != NULL; app_it = app_it->next) {
				WnckApplication* _tmp3_;
				WnckApplication* app;
				_tmp3_ = NULL;
				app = (_tmp3_ = (WnckApplication*) app_it->data, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_));
				{
					all_tray_application_maybe_setup (self, all_tray_program_WnckScreen, app);
					(app == NULL) ? NULL : (app = (g_object_unref (app), NULL));
				}
			}
		}
	}
	return self;
}


AllTrayApplication* all_tray_application_new (AllTrayProcess* p) {
	return all_tray_application_construct (ALL_TRAY_TYPE_APPLICATION, p);
}


static void _all_tray_application_maybe_update_window_count_wnck_screen_window_opened (WnckScreen* _sender, WnckWindow* window, gpointer self) {
	all_tray_application_maybe_update_window_count (self, _sender, window);
}


static void _all_tray_application_maybe_update_window_count_wnck_screen_window_closed (WnckScreen* _sender, WnckWindow* window, gpointer self) {
	all_tray_application_maybe_update_window_count (self, _sender, window);
}


static void all_tray_application_bye_wnck_app (AllTrayApplication* self, WnckScreen* scr, WnckApplication* app) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (scr != NULL);
	g_return_if_fail (app != NULL);
	if (app == self->priv->_wnckApp) {
		guint _tmp0_;
		guint _tmp1_;
		all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Application, ALL_TRAY_DEBUG_LEVEL_Information, _ ("WHOA - The app went away?! Looking for it to come back..."));
		g_signal_handlers_disconnect_matched (scr, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("window-opened", WNCK_TYPE_SCREEN, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _all_tray_application_maybe_update_window_count_wnck_screen_window_opened, self);
		g_signal_handlers_disconnect_matched (scr, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("window-closed", WNCK_TYPE_SCREEN, &_tmp1_, NULL, FALSE), _tmp1_), 0, NULL, (GCallback) _all_tray_application_maybe_update_window_count_wnck_screen_window_closed, self);
		g_signal_connect_object (all_tray_program_WnckScreen, "application-opened", (GCallback) _all_tray_application_maybe_setup_wnck_screen_application_opened, self, 0);
	}
}


static gboolean all_tray_application_are_we_interested (AllTrayApplication* self, gint pid) {
	gboolean result;
	GError * _inner_error_;
	gboolean retval;
	gint app_pid;
	gint app_ppid;
	gint app_pgid;
	gint desired_pid;
	gint desired_pgid;
	char* proc_name;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	_inner_error_ = NULL;
	g_return_val_if_fail (pid != 0, FALSE);
	retval = FALSE;
	app_pid = pid;
	app_ppid = alltray_get_ppid_for (app_pid);
	app_pgid = (gint) getpgid ((pid_t) app_pid);
	desired_pid = (gint) all_tray_process_get_pid (self->priv->_process);
	desired_pgid = (gint) all_tray_program_pgid;
	proc_name = alltray_get_process_name (pid);
	_tmp0_ = FALSE;
	if (app_pgid == desired_pgid) {
		_tmp0_ = app_pgid != app_pid;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		retval = TRUE;
	} else {
		if (app_pid == desired_pid) {
			retval = TRUE;
		} else {
			if (app_ppid == desired_pid) {
				retval = TRUE;
			}
		}
	}
	if (!retval) {
		GString* msg;
		msg = g_string_new ("");
		g_string_append_printf (msg, _ ("pid %d (%s) is not ours."), app_pid, proc_name);
		all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Application, ALL_TRAY_DEBUG_LEVEL_Information, msg->str);
		(msg == NULL) ? NULL : (msg = (g_string_free (msg, TRUE), NULL));
	} else {
		if (app_pid != desired_pid) {
			all_tray_process_run_kinda_fake (self->priv->_process, app_pid, &_inner_error_);
			if (_inner_error_ != NULL) {
				proc_name = (g_free (proc_name), NULL);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return FALSE;
			}
		}
	}
	result = retval;
	proc_name = (g_free (proc_name), NULL);
	return result;
}


void all_tray_application_maybe_setup_for_pid (AllTrayApplication* self, WnckApplication* app, gint pid) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (app != NULL);
	if (pid == 0) {
		g_warning (_ ("Application %s needs fixed, reports PID 0."), wnck_application_get_name (app));
	} else {
		gboolean interested;
		interested = self->priv->_caughtWindow = all_tray_application_are_we_interested (self, pid);
		if (!interested) {
			return;
		}
		all_tray_application_do_setup (self, all_tray_program_WnckScreen, app, pid);
	}
}


static void all_tray_application_maybe_setup (AllTrayApplication* self, WnckScreen* scr, WnckApplication* app) {
	gboolean interested;
	g_return_if_fail (self != NULL);
	g_return_if_fail (scr != NULL);
	g_return_if_fail (app != NULL);
	interested = self->priv->_caughtWindow = all_tray_application_are_we_interested (self, wnck_application_get_pid (app));
	if (!interested) {
		return;
	} else {
		gboolean _tmp0_;
		_tmp0_ = FALSE;
		if (interested) {
			_tmp0_ = all_tray_process_get_timer_running (self->priv->_process);
		} else {
			_tmp0_ = FALSE;
		}
		if (_tmp0_) {
			/* Prevent AllTray from blowing itself up when an app shows up*/
			g_source_remove (all_tray_process_get_timer_event_source (self->priv->_process));
		}
	}
	all_tray_application_do_setup (self, scr, app, wnck_application_get_pid (app));
}


static void _all_tray_application_display_menu_gtk_status_icon_popup_menu (GtkStatusIcon* _sender, guint button, guint activate_time, gpointer self) {
	all_tray_application_display_menu (self, button, activate_time);
}


static void all_tray_application_do_setup (AllTrayApplication* self, WnckScreen* scr, WnckApplication* app, gint pid) {
	guint _tmp0_;
	WnckApplication* _tmp2_;
	WnckApplication* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (scr != NULL);
	g_return_if_fail (app != NULL);
	g_signal_handlers_disconnect_matched (scr, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("application-opened", WNCK_TYPE_SCREEN, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _all_tray_application_maybe_setup_wnck_screen_application_opened, self);
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	self->priv->_wnckApp = (_tmp2_ = (_tmp1_ = app, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (self->priv->_wnckApp == NULL) ? NULL : (self->priv->_wnckApp = (g_object_unref (self->priv->_wnckApp), NULL)), _tmp2_);
	self->priv->_appVisible = TRUE;
	self->priv->_windows = wnck_application_get_windows (self->priv->_wnckApp);
	g_signal_connect_object (scr, "window-opened", (GCallback) _all_tray_application_maybe_update_window_count_wnck_screen_window_opened, self, 0);
	g_signal_connect_object (scr, "window-closed", (GCallback) _all_tray_application_maybe_update_window_count_wnck_screen_window_closed, self, 0);
	if (self->priv->_appIcon == NULL) {
		all_tray_application_create_icon (self);
	}
	gtk_status_icon_set_visible (self->priv->_appIcon, TRUE);
	g_signal_connect_object (self->priv->_appIcon, "popup-menu", (GCallback) _all_tray_application_display_menu_gtk_status_icon_popup_menu, self, 0);
	/* Force an update to catch circumstances where the app
	 disappears and reappears (e.g., The GIMP).*/
	if (self->priv->_usingWindowIcon) {
		all_tray_application_update_icon_win_image (self, (WnckWindow*) g_list_first (self->priv->_windows)->data);
	} else {
		all_tray_application_update_icon_app_image (self, self->priv->_wnckApp);
	}
	if (all_tray_program__initially_hide == TRUE) {
		all_tray_application_hide_all_windows (self);
	}
}


static void _all_tray_application_on_menu_undock_gtk_menu_item_activate (GtkMenuItem* _sender, gpointer self) {
	all_tray_application_on_menu_undock (self);
}


static void _all_tray_application_on_menu_about_gtk_menu_item_activate (GtkMenuItem* _sender, gpointer self) {
	all_tray_application_on_menu_about (self);
}


static void all_tray_application_display_menu (AllTrayApplication* self, guint button, guint activate_time) {
	GtkMenu* appMenu;
	GtkMenuItem* mnuToggle;
	GtkMenuItem* mnuUndock;
	GtkMenuItem* mnuSeparator0;
	GtkMenuItem* mnuAbout;
	GtkMenu* windowList;
	g_return_if_fail (self != NULL);
	all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Application, ALL_TRAY_DEBUG_LEVEL_Information, _ ("Requesting menu!"));
	appMenu = g_object_ref_sink ((GtkMenu*) gtk_menu_new ());
	mnuToggle = g_object_ref_sink ((GtkMenuItem*) gtk_menu_item_new_with_label (_ ("Toggle Visibility")));
	mnuUndock = g_object_ref_sink ((GtkMenuItem*) gtk_menu_item_new_with_label (_ ("Undock")));
	mnuSeparator0 = (GtkMenuItem*) g_object_ref_sink ((GtkSeparatorMenuItem*) gtk_separator_menu_item_new ());
	mnuAbout = g_object_ref_sink ((GtkMenuItem*) gtk_menu_item_new_with_label (_ ("About AllTray...")));
	windowList = all_tray_application_create_window_list_menu (self);
	gtk_menu_item_set_submenu (mnuToggle, (GtkWidget*) windowList);
	/* Append, wire and show the menu items.*/
	gtk_menu_shell_append ((GtkMenuShell*) appMenu, (GtkWidget*) mnuToggle);
	gtk_widget_show ((GtkWidget*) mnuToggle);
	gtk_menu_shell_append ((GtkMenuShell*) appMenu, (GtkWidget*) mnuUndock);
	g_signal_connect_object (mnuUndock, "activate", (GCallback) _all_tray_application_on_menu_undock_gtk_menu_item_activate, self, 0);
	gtk_widget_show ((GtkWidget*) mnuUndock);
	gtk_menu_shell_append ((GtkMenuShell*) appMenu, (GtkWidget*) mnuSeparator0);
	gtk_widget_show ((GtkWidget*) mnuSeparator0);
	gtk_menu_shell_append ((GtkMenuShell*) appMenu, (GtkWidget*) mnuAbout);
	g_signal_connect_object (mnuAbout, "activate", (GCallback) _all_tray_application_on_menu_about_gtk_menu_item_activate, self, 0);
	gtk_widget_show ((GtkWidget*) mnuAbout);
	gtk_menu_popup (appMenu, NULL, NULL, NULL, NULL, button, (guint32) activate_time);
	(appMenu == NULL) ? NULL : (appMenu = (g_object_unref (appMenu), NULL));
	(mnuToggle == NULL) ? NULL : (mnuToggle = (g_object_unref (mnuToggle), NULL));
	(mnuUndock == NULL) ? NULL : (mnuUndock = (g_object_unref (mnuUndock), NULL));
	(mnuSeparator0 == NULL) ? NULL : (mnuSeparator0 = (g_object_unref (mnuSeparator0), NULL));
	(mnuAbout == NULL) ? NULL : (mnuAbout = (g_object_unref (mnuAbout), NULL));
	(windowList == NULL) ? NULL : (windowList = (g_object_unref (windowList), NULL));
}


static void _all_tray_application_on_menu_toggle_app_gtk_menu_item_activate (GtkMenuItem* _sender, gpointer self) {
	all_tray_application_on_menu_toggle_app (self, _sender);
}


static void _lambda0_ (GtkMenuItem* item, AllTrayApplication* self) {
	WnckWindow* _tmp0_;
	WnckWindow* win;
	g_return_if_fail (item != NULL);
	_tmp0_ = NULL;
	win = (_tmp0_ = WNCK_WINDOW (g_object_get_data ((GObject*) item, "target_window")), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	all_tray_application_toggle_window_visibility (self, win);
	(win == NULL) ? NULL : (win = (g_object_unref (win), NULL));
}


static void __lambda0__gtk_menu_item_activate (GtkMenuItem* _sender, gpointer self) {
	_lambda0_ (_sender, self);
}


static GtkMenu* all_tray_application_create_window_list_menu (AllTrayApplication* self) {
	GtkMenu* result;
	GtkMenu* retval;
	GtkMenuItem* mnuAllWindows;
	GtkMenuItem* mnuSep0;
	g_return_val_if_fail (self != NULL, NULL);
	retval = g_object_ref_sink ((GtkMenu*) gtk_menu_new ());
	mnuAllWindows = g_object_ref_sink ((GtkMenuItem*) gtk_menu_item_new_with_label (_ ("All")));
	mnuSep0 = (GtkMenuItem*) g_object_ref_sink ((GtkSeparatorMenuItem*) gtk_separator_menu_item_new ());
	gtk_menu_shell_append ((GtkMenuShell*) retval, (GtkWidget*) mnuAllWindows);
	g_signal_connect_object (mnuAllWindows, "activate", (GCallback) _all_tray_application_on_menu_toggle_app_gtk_menu_item_activate, self, 0);
	gtk_widget_show ((GtkWidget*) mnuAllWindows);
	gtk_menu_shell_append ((GtkMenuShell*) retval, (GtkWidget*) mnuSep0);
	gtk_widget_show ((GtkWidget*) mnuSep0);
	{
		GList* w_collection;
		GList* w_it;
		w_collection = self->priv->_windows;
		for (w_it = w_collection; w_it != NULL; w_it = w_it->next) {
			WnckWindow* _tmp0_;
			WnckWindow* w;
			_tmp0_ = NULL;
			w = (_tmp0_ = (WnckWindow*) w_it->data, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
			{
				GtkMenuItem* item;
				item = g_object_ref_sink ((GtkMenuItem*) gtk_menu_item_new_with_label (wnck_window_get_name (w)));
				g_object_set_data ((GObject*) item, "target_window", w);
				g_signal_connect (item, "activate", (GCallback) __lambda0__gtk_menu_item_activate, self);
				gtk_menu_shell_append ((GtkMenuShell*) retval, (GtkWidget*) item);
				gtk_widget_show ((GtkWidget*) item);
				(w == NULL) ? NULL : (w = (g_object_unref (w), NULL));
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			}
		}
	}
	result = retval;
	(mnuAllWindows == NULL) ? NULL : (mnuAllWindows = (g_object_unref (mnuAllWindows), NULL));
	(mnuSep0 == NULL) ? NULL : (mnuSep0 = (g_object_unref (mnuSep0), NULL));
	return result;
}


static void all_tray_application_maybe_update_window_count (AllTrayApplication* self, WnckScreen* scr, WnckWindow* win) {
	GString* sb;
	gint wincount;
	const char* _tmp0_;
	const char* _tmp1_;
	char* plural;
	g_return_if_fail (self != NULL);
	g_return_if_fail (scr != NULL);
	g_return_if_fail (win != NULL);
	sb = g_string_new ("");
	self->priv->_windows = wnck_application_get_windows (self->priv->_wnckApp);
	wincount = wnck_application_get_n_windows (self->priv->_wnckApp);
	_tmp0_ = NULL;
	if (wincount != 1) {
		_tmp0_ = "s";
	} else {
		_tmp0_ = "";
	}
	_tmp1_ = NULL;
	plural = (_tmp1_ = _tmp0_, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
	g_string_append_printf (sb, ngettext ("%s - %d window", "%s - %d windows", (gulong) wincount), wnck_application_get_name (self->priv->_wnckApp), wincount);
	gtk_status_icon_set_tooltip (self->priv->_appIcon, sb->str);
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
	plural = (g_free (plural), NULL);
}


static void _all_tray_application_update_icon_win_image_wnck_window_icon_changed (WnckWindow* _sender, gpointer self) {
	all_tray_application_update_icon_win_image (self, _sender);
}


static void _all_tray_application_update_icon_app_image_wnck_application_icon_changed (WnckApplication* _sender, gpointer self) {
	all_tray_application_update_icon_app_image (self, _sender);
}


static void _all_tray_application_on_icon_click_gtk_status_icon_activate (GtkStatusIcon* _sender, gpointer self) {
	all_tray_application_on_icon_click (self, _sender);
}


static void _all_tray_application_update_icon_name_wnck_application_name_changed (WnckApplication* _sender, gpointer self) {
	all_tray_application_update_icon_name (self, _sender);
}


static char* bool_to_string (gboolean self) {
	char* result;
	if (self) {
		result = g_strdup ("true");
		return result;
	} else {
		result = g_strdup ("false");
		return result;
	}
}


/*
     * This tries to create an icon for the application.  First, we
     * see if the application has an icon (don't know *precisely* what
     * this means from Wnck's perspective... clarification would be
     * useful, as I don't have time to look at their source ATM).  If
     * the application does not have an icon, then we look to see if
     * the window does.
     *
     * If not, then we just use a fallback.
     *
     * Note that this only selects the first window it finds.  There
     * may be a better way to do this; time will tell.
     */
static void all_tray_application_create_icon (AllTrayApplication* self) {
	WnckWindow* _tmp0_;
	WnckWindow* firstWindow;
	gboolean fallback;
	char* msg;
	g_return_if_fail (self != NULL);
	self->priv->_windows = wnck_application_get_windows (self->priv->_wnckApp);
	_tmp0_ = NULL;
	firstWindow = (_tmp0_ = (WnckWindow*) g_list_first (self->priv->_windows)->data, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	self->priv->_usingWindowIcon = FALSE;
	fallback = wnck_application_get_icon_is_fallback (self->priv->_wnckApp);
	if (fallback) {
		GtkStatusIcon* _tmp1_;
		_tmp1_ = NULL;
		self->priv->_appIcon = (_tmp1_ = gtk_status_icon_new_from_pixbuf (wnck_window_get_mini_icon (firstWindow)), (self->priv->_appIcon == NULL) ? NULL : (self->priv->_appIcon = (g_object_unref (self->priv->_appIcon), NULL)), _tmp1_);
		g_signal_connect_object (firstWindow, "icon-changed", (GCallback) _all_tray_application_update_icon_win_image_wnck_window_icon_changed, self, 0);
		self->priv->_usingWindowIcon = TRUE;
	} else {
		GtkStatusIcon* _tmp2_;
		_tmp2_ = NULL;
		self->priv->_appIcon = (_tmp2_ = gtk_status_icon_new_from_pixbuf (wnck_application_get_mini_icon (self->priv->_wnckApp)), (self->priv->_appIcon == NULL) ? NULL : (self->priv->_appIcon = (g_object_unref (self->priv->_appIcon), NULL)), _tmp2_);
		g_signal_connect_object (self->priv->_wnckApp, "icon-changed", (GCallback) _all_tray_application_update_icon_app_image_wnck_application_icon_changed, self, 0);
	}
	gtk_status_icon_set_tooltip (self->priv->_appIcon, wnck_application_get_name (self->priv->_wnckApp));
	g_signal_connect_object (self->priv->_appIcon, "activate", (GCallback) _all_tray_application_on_icon_click_gtk_status_icon_activate, self, 0);
	g_signal_connect_object (self->priv->_wnckApp, "name-changed", (GCallback) _all_tray_application_update_icon_name_wnck_application_name_changed, self, 0);
	msg = g_strdup ("");
	if (self->priv->_usingWindowIcon) {
		char* _tmp4_;
		char* _tmp3_;
		_tmp4_ = NULL;
		_tmp3_ = NULL;
		msg = (_tmp4_ = g_strdup_printf (_ ("Using Window Icon (fallback was %s)"), _tmp3_ = bool_to_string (fallback)), msg = (g_free (msg), NULL), _tmp4_);
		_tmp3_ = (g_free (_tmp3_), NULL);
	} else {
		char* _tmp6_;
		char* _tmp5_;
		_tmp6_ = NULL;
		_tmp5_ = NULL;
		msg = (_tmp6_ = g_strdup_printf (_ ("Using App Icon (fallback was %s)"), _tmp5_ = bool_to_string (fallback)), msg = (g_free (msg), NULL), _tmp6_);
		_tmp5_ = (g_free (_tmp5_), NULL);
	}
	all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Application, ALL_TRAY_DEBUG_LEVEL_Information, msg);
	(firstWindow == NULL) ? NULL : (firstWindow = (g_object_unref (firstWindow), NULL));
	msg = (g_free (msg), NULL);
}


static void all_tray_application_update_icon_app_image (AllTrayApplication* self, WnckApplication* app) {
	GdkPixbuf* new_icon;
	g_return_if_fail (self != NULL);
	g_return_if_fail (app != NULL);
	new_icon = wnck_application_get_icon (app);
	gtk_status_icon_set_from_pixbuf (self->priv->_appIcon, new_icon);
	if (new_icon == NULL) {
		gtk_status_icon_set_visible (self->priv->_appIcon, FALSE);
		return;
	}
}


static void all_tray_application_update_icon_win_image (AllTrayApplication* self, WnckWindow* win) {
	GdkPixbuf* new_icon;
	g_return_if_fail (self != NULL);
	g_return_if_fail (win != NULL);
	new_icon = wnck_window_get_mini_icon (win);
	gtk_status_icon_set_from_pixbuf (self->priv->_appIcon, new_icon);
	if (new_icon == NULL) {
		gtk_status_icon_set_visible (self->priv->_appIcon, FALSE);
		return;
	}
}


static void all_tray_application_update_icon_name (AllTrayApplication* self, WnckApplication* app) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (app != NULL);
	gtk_status_icon_set_tooltip (self->priv->_appIcon, wnck_application_get_name (app));
}


static void all_tray_application_on_icon_click (AllTrayApplication* self, GtkStatusIcon* icon) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (icon != NULL);
	if (gtk_status_icon_get_blinking (icon) == TRUE) {
		gtk_status_icon_set_blinking (icon, FALSE);
	}
	all_tray_application_toggle_visibility (self);
}


/*
     * This looks like it works.  However, the user may notice the
     * window trying to appear for a moment before we catch it...
     *
     * TODO: Find a way to intercept the window activation and
     * prevent that from happening at all, and instead setting the
     * DEMANDS_ATTENTION window manager hint, and generally blink
     * the icon on DEMANDS_ATTENTION instead.
     */
static void all_tray_application_maintain_hiddenness (AllTrayApplication* self, WnckWindow* win, WnckWindowState changed_bits, WnckWindowState new_state) {
	char* _tmp0_;
	char* _tmp1_;
	char* ch_bitmask;
	char* _tmp2_;
	char* _tmp3_;
	char* ch_new_state;
	gboolean _tmp4_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (win != NULL);
	if (self->priv->_appVisible) {
		return;
	}
	_tmp0_ = NULL;
	_tmp1_ = NULL;
	ch_bitmask = (_tmp1_ = g_strdup_printf (_ ("New Bitmask: %s"), _tmp0_ = all_tray_application_get_win_state (self, changed_bits)), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	_tmp2_ = NULL;
	_tmp3_ = NULL;
	ch_new_state = (_tmp3_ = g_strdup_printf (_ ("New State: %s"), _tmp2_ = all_tray_application_get_win_state (self, new_state)), _tmp2_ = (g_free (_tmp2_), NULL), _tmp3_);
	all_tray_application_debug_msg (self, ch_bitmask);
	all_tray_application_debug_msg (self, ch_new_state);
	_tmp4_ = FALSE;
	if ((new_state & WNCK_WINDOW_STATE_MINIMIZED) == 0) {
		_tmp4_ = (changed_bits & WNCK_WINDOW_STATE_MINIMIZED) == WNCK_WINDOW_STATE_MINIMIZED;
	} else {
		_tmp4_ = FALSE;
	}
	if (_tmp4_) {
		char* _tmp6_;
		char* _tmp5_;
		_tmp6_ = NULL;
		_tmp5_ = NULL;
		all_tray_application_debug_msg (self, _tmp6_ = g_strdup_printf (_ ("Blinking, window state is: %s"), _tmp5_ = all_tray_application_get_win_state (self, new_state)));
		_tmp6_ = (g_free (_tmp6_), NULL);
		_tmp5_ = (g_free (_tmp5_), NULL);
		wnck_window_minimize (win);
		gtk_status_icon_set_blinking (self->priv->_appIcon, TRUE);
	}
	ch_bitmask = (g_free (ch_bitmask), NULL);
	ch_new_state = (g_free (ch_new_state), NULL);
}


static void all_tray_application_debug_msg (AllTrayApplication* self, const char* str) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (str != NULL);
	all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Application, ALL_TRAY_DEBUG_LEVEL_Information, str);
}


static char* all_tray_application_get_win_state (AllTrayApplication* self, WnckWindowState state) {
	char* result;
	GString* sb;
	const char* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	sb = g_string_new ("");
	if ((state & WNCK_WINDOW_STATE_MINIMIZED) != 0) {
		g_string_append (sb, _ ("MINIMIZED | "));
	}
	if ((state & WNCK_WINDOW_STATE_MAXIMIZED_HORIZONTALLY) != 0) {
		g_string_append (sb, _ ("MAXIMIZED_HORIZONTALLY | "));
	}
	if ((state & WNCK_WINDOW_STATE_MAXIMIZED_VERTICALLY) != 0) {
		g_string_append (sb, _ ("MAXIMIZED_VERTICALLY | "));
	}
	if ((state & WNCK_WINDOW_STATE_SHADED) != 0) {
		g_string_append (sb, _ ("SHADED | "));
	}
	if ((state & WNCK_WINDOW_STATE_SKIP_PAGER) != 0) {
		g_string_append (sb, _ ("SKIP_PAGER | "));
	}
	if ((state & WNCK_WINDOW_STATE_SKIP_TASKLIST) != 0) {
		g_string_append (sb, _ ("SKIP_TASKLIST | "));
	}
	if ((state & WNCK_WINDOW_STATE_STICKY) != 0) {
		g_string_append (sb, _ ("STICKY | "));
	}
	if ((state & WNCK_WINDOW_STATE_HIDDEN) != 0) {
		g_string_append (sb, _ ("HIDDEN | "));
	}
	if ((state & WNCK_WINDOW_STATE_FULLSCREEN) != 0) {
		g_string_append (sb, _ ("FULLSCREEN | "));
	}
	if ((state & WNCK_WINDOW_STATE_DEMANDS_ATTENTION) != 0) {
		g_string_append (sb, _ ("DEMANDS_ATTENTION | "));
	}
	if ((state & WNCK_WINDOW_STATE_URGENT) != 0) {
		g_string_append (sb, _ ("URGENT | "));
	}
	if ((state & WNCK_WINDOW_STATE_ABOVE) != 0) {
		g_string_append (sb, _ ("ABOVE | "));
	}
	if ((state & WNCK_WINDOW_STATE_BELOW) != 0) {
		g_string_append (sb, _ ("BELOW | "));
	}
	g_string_truncate (sb, (gsize) (sb->len - 3));
	_tmp0_ = NULL;
	result = (_tmp0_ = sb->str, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
	return result;
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
}


static void all_tray_application_toggle_visibility (AllTrayApplication* self) {
	g_return_if_fail (self != NULL);
	self->priv->_appVisible = !self->priv->_appVisible;
	{
		GList* w_collection;
		GList* w_it;
		w_collection = self->priv->_windows;
		for (w_it = w_collection; w_it != NULL; w_it = w_it->next) {
			WnckWindow* _tmp0_;
			WnckWindow* w;
			_tmp0_ = NULL;
			w = (_tmp0_ = (WnckWindow*) w_it->data, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
			{
				all_tray_application_set_visibility_for_window (self, w, self->priv->_appVisible);
				(w == NULL) ? NULL : (w = (g_object_unref (w), NULL));
			}
		}
	}
}


static void all_tray_application_show_all_windows (AllTrayApplication* self) {
	g_return_if_fail (self != NULL);
	self->priv->_appVisible = FALSE;
	all_tray_application_toggle_visibility (self);
}


static void all_tray_application_hide_all_windows (AllTrayApplication* self) {
	g_return_if_fail (self != NULL);
	self->priv->_appVisible = TRUE;
	all_tray_application_toggle_visibility (self);
}


static void all_tray_application_toggle_window_visibility (AllTrayApplication* self, WnckWindow* w) {
	WnckWindowState ws;
	gboolean _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (w != NULL);
	ws = wnck_window_get_state (w);
	_tmp0_ = FALSE;
	if ((ws & WNCK_WINDOW_STATE_MINIMIZED) != 0) {
		_tmp0_ = (ws & WNCK_WINDOW_STATE_SKIP_TASKLIST) != 0;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		all_tray_application_set_visibility_for_window (self, w, TRUE);
	} else {
		all_tray_application_set_visibility_for_window (self, w, FALSE);
	}
}


static void _all_tray_application_maintain_hiddenness_wnck_window_state_changed (WnckWindow* _sender, WnckWindowState changed_mask, WnckWindowState new_state, gpointer self) {
	all_tray_application_maintain_hiddenness (self, _sender, changed_mask, new_state);
}


static gboolean _all_tray_application_set_maintain_hiddenness_gsource_func (gpointer self) {
	return all_tray_application_set_maintain_hiddenness (self);
}


static void all_tray_application_set_visibility_for_window (AllTrayApplication* self, WnckWindow* w, gboolean set_visible) {
	GTimeVal _tmp0_ = {0};
	GTimeVal tv;
	GString* msg;
	char* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (w != NULL);
	tv = (g_get_current_time (&_tmp0_), _tmp0_);
	g_get_current_time (&tv);
	msg = g_string_new ("");
	_tmp1_ = NULL;
	g_string_append_printf (msg, _ ("Setting window 0x%08lx visibility to %s"), wnck_window_get_xid (w), _tmp1_ = bool_to_string (self->priv->_appVisible));
	_tmp1_ = (g_free (_tmp1_), NULL);
	if (set_visible) {
		guint _tmp2_;
		WnckWorkspace* _tmp3_;
		WnckWorkspace* ws;
		/* Forcibly empty the queue (prevent a race).*/
		g_queue_clear (self->priv->_window_enforce_minimize_queue);
		g_signal_handlers_disconnect_matched (w, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("state-changed", WNCK_TYPE_WINDOW, &_tmp2_, NULL, FALSE), _tmp2_), 0, NULL, (GCallback) _all_tray_application_maintain_hiddenness_wnck_window_state_changed, self);
		wnck_window_set_skip_tasklist (w, FALSE);
		wnck_window_set_skip_pager (w, FALSE);
		wnck_window_unminimize (w, (guint32) tv.tv_sec);
		_tmp3_ = NULL;
		ws = (_tmp3_ = wnck_window_get_workspace (w), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_));
		wnck_workspace_activate (ws, (guint32) tv.tv_sec);
		wnck_window_activate (w, (guint32) tv.tv_sec);
		(ws == NULL) ? NULL : (ws = (g_object_unref (ws), NULL));
	} else {
		WnckWindow* _tmp4_;
		wnck_window_set_skip_tasklist (w, TRUE);
		wnck_window_set_skip_pager (w, TRUE);
		wnck_window_minimize (w);
		_tmp4_ = NULL;
		g_queue_push_head (self->priv->_window_enforce_minimize_queue, (_tmp4_ = w, (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_)));
		g_timeout_add ((guint) 150, _all_tray_application_set_maintain_hiddenness_gsource_func, self);
	}
	all_tray_debug_notification_emit (ALL_TRAY_DEBUG_SUBSYSTEM_Application, ALL_TRAY_DEBUG_LEVEL_Information, msg->str);
	(msg == NULL) ? NULL : (msg = (g_string_free (msg, TRUE), NULL));
}


static gboolean all_tray_application_set_maintain_hiddenness (AllTrayApplication* self) {
	gboolean result;
	WnckWindow* w;
	g_return_val_if_fail (self != NULL, FALSE);
	w = NULL;
	while (TRUE) {
		WnckWindow* _tmp0_;
		_tmp0_ = NULL;
		if (!((w = (_tmp0_ = (WnckWindow*) g_queue_pop_tail (self->priv->_window_enforce_minimize_queue), (w == NULL) ? NULL : (w = (g_object_unref (w), NULL)), _tmp0_)) != NULL)) {
			break;
		}
		g_signal_connect_object (w, "state-changed", (GCallback) _all_tray_application_maintain_hiddenness_wnck_window_state_changed, self, 0);
	}
	/*
	       * Only call us again if the queue has had things pop up in
	       * it that we didn't handle this time.
	       */
	result = self->priv->_window_enforce_minimize_queue->length == 0;
	(w == NULL) ? NULL : (w = (g_object_unref (w), NULL));
	return result;
}


static void all_tray_application_dialog_destroy (AllTrayApplication* self, GtkAboutDialog* which, gint resp_id) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (which != NULL);
	gtk_object_destroy ((GtkObject*) which);
}


/**********************************************************
     * Context menu handlers.
     **********************************************************/
static void all_tray_application_on_menu_toggle_app (AllTrayApplication* self, GtkMenuItem* item) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (item != NULL);
	if (gtk_status_icon_get_blinking (self->priv->_appIcon) == TRUE) {
		gtk_status_icon_set_blinking (self->priv->_appIcon, FALSE);
	}
	all_tray_application_toggle_visibility (self);
}


static void _all_tray_application_dialog_destroy_gtk_dialog_response (GtkAboutDialog* _sender, gint response_id, gpointer self) {
	all_tray_application_dialog_destroy (self, _sender, response_id);
}


static void all_tray_application_on_menu_about (AllTrayApplication* self) {
	GtkAboutDialog* about;
	gboolean _tmp0_;
	char* _tmp2_;
	g_return_if_fail (self != NULL);
	/* Display an about dialog*/
	about = g_object_ref_sink ((GtkAboutDialog*) gtk_about_dialog_new ());
	gtk_about_dialog_set_program_name (about, "AllTray");
	_tmp0_ = FALSE;
	if (g_utf8_strchr (PACKAGE_VERSION, (gssize) (-1), (gunichar) '+') != NULL) {
		_tmp0_ = _vala_strcmp0 (ALLTRAY_BZR_BUILD, "TRUE") == 0;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		char* _tmp1_;
		_tmp1_ = NULL;
		gtk_about_dialog_set_version (about, _tmp1_ = g_strdup_printf (_ ("%s\nBranch: %s, r%s\n%s"), PACKAGE_VERSION, ALLTRAY_BZR_BRANCH, ALLTRAY_BZR_REVISION, ALLTRAY_BZR_REVID));
		_tmp1_ = (g_free (_tmp1_), NULL);
	} else {
		gtk_about_dialog_set_version (about, PACKAGE_VERSION);
	}
	gtk_about_dialog_set_website (about, "http://alltray.trausch.us/");
	_tmp2_ = NULL;
	gtk_about_dialog_set_copyright (about, _tmp2_ = g_strdup_printf (_ ("Copyright (c) %s"), ALLTRAY_COPYRIGHT_YEARS));
	_tmp2_ = (g_free (_tmp2_), NULL);
	gtk_about_dialog_set_comments (about, _ ("Dock applications in the system tray."));
	gtk_about_dialog_set_license (about, ALLTRAY_LICENSE);
	g_signal_connect_object ((GtkDialog*) about, "response", (GCallback) _all_tray_application_dialog_destroy_gtk_dialog_response, self, 0);
	gtk_widget_show_all ((GtkWidget*) about);
	(about == NULL) ? NULL : (about = (g_object_unref (about), NULL));
}


static void all_tray_application_on_menu_undock (AllTrayApplication* self) {
	g_return_if_fail (self != NULL);
	all_tray_application_show_all_windows (self);
	exit (0);
}


WnckApplication* all_tray_application_get_wnck_app (AllTrayApplication* self) {
	WnckApplication* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_wnckApp;
	return result;
}


gboolean all_tray_application_get_visible (AllTrayApplication* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_appVisible;
	return result;
}


void all_tray_application_set_visible (AllTrayApplication* self, gboolean value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_appVisible != value) {
		all_tray_application_toggle_visibility (self);
	}
	g_object_notify ((GObject *) self, "visible");
}


gboolean all_tray_application_get_caught_window (AllTrayApplication* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = self->priv->_caughtWindow;
	return result;
}


void all_tray_application_set_caught_window (AllTrayApplication* self, gboolean value) {
	g_return_if_fail (self != NULL);
	if (self->priv->_caughtWindow) {
		return;
	}
	if (value == FALSE) {
		return;
	}
	self->priv->_caughtWindow = value;
	g_object_notify ((GObject *) self, "caught-window");
}


static void all_tray_application_class_init (AllTrayApplicationClass * klass) {
	all_tray_application_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (AllTrayApplicationPrivate));
	G_OBJECT_CLASS (klass)->get_property = all_tray_application_get_property;
	G_OBJECT_CLASS (klass)->set_property = all_tray_application_set_property;
	G_OBJECT_CLASS (klass)->finalize = all_tray_application_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), ALL_TRAY_APPLICATION_WNCK_APP, g_param_spec_object ("wnck-app", "wnck-app", "wnck-app", WNCK_TYPE_APPLICATION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), ALL_TRAY_APPLICATION_VISIBLE, g_param_spec_boolean ("visible", "visible", "visible", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), ALL_TRAY_APPLICATION_CAUGHT_WINDOW, g_param_spec_boolean ("caught-window", "caught-window", "caught-window", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void all_tray_application_instance_init (AllTrayApplication * self) {
	self->priv = ALL_TRAY_APPLICATION_GET_PRIVATE (self);
}


static void all_tray_application_finalize (GObject* obj) {
	AllTrayApplication * self;
	self = ALL_TRAY_APPLICATION (obj);
	(self->priv->_wnckApp == NULL) ? NULL : (self->priv->_wnckApp = (g_object_unref (self->priv->_wnckApp), NULL));
	(self->priv->_appIcon == NULL) ? NULL : (self->priv->_appIcon = (g_object_unref (self->priv->_appIcon), NULL));
	(self->priv->_process == NULL) ? NULL : (self->priv->_process = (g_object_unref (self->priv->_process), NULL));
	(self->priv->_window_enforce_minimize_queue == NULL) ? NULL : (self->priv->_window_enforce_minimize_queue = (g_queue_free (self->priv->_window_enforce_minimize_queue), NULL));
	G_OBJECT_CLASS (all_tray_application_parent_class)->finalize (obj);
}


GType all_tray_application_get_type (void) {
	static GType all_tray_application_type_id = 0;
	if (all_tray_application_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (AllTrayApplicationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) all_tray_application_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AllTrayApplication), 0, (GInstanceInitFunc) all_tray_application_instance_init, NULL };
		all_tray_application_type_id = g_type_register_static (G_TYPE_OBJECT, "AllTrayApplication", &g_define_type_info, 0);
	}
	return all_tray_application_type_id;
}


static void all_tray_application_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	AllTrayApplication * self;
	gpointer boxed;
	self = ALL_TRAY_APPLICATION (object);
	switch (property_id) {
		case ALL_TRAY_APPLICATION_WNCK_APP:
		g_value_set_object (value, all_tray_application_get_wnck_app (self));
		break;
		case ALL_TRAY_APPLICATION_VISIBLE:
		g_value_set_boolean (value, all_tray_application_get_visible (self));
		break;
		case ALL_TRAY_APPLICATION_CAUGHT_WINDOW:
		g_value_set_boolean (value, all_tray_application_get_caught_window (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void all_tray_application_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	AllTrayApplication * self;
	self = ALL_TRAY_APPLICATION (object);
	switch (property_id) {
		case ALL_TRAY_APPLICATION_VISIBLE:
		all_tray_application_set_visible (self, g_value_get_boolean (value));
		break;
		case ALL_TRAY_APPLICATION_CAUGHT_WINDOW:
		all_tray_application_set_caught_window (self, g_value_get_boolean (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




