/* -*- mode: vala; c-basic-offset: 2; tab-width: 8; -*-
 * PromptDialog.vala - Dialog window to prompt the user to select an app
 * Copyright (c) 2009 Michael B. Trausch <mike@trausch.us>
 * License: GNU GPL v3.0 as published by the Free Software Foundation
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>
#include <stdlib.h>
#include <string.h>


#define ALL_TRAY_TYPE_PROMPT_DIALOG (all_tray_prompt_dialog_get_type ())
#define ALL_TRAY_PROMPT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ALL_TRAY_TYPE_PROMPT_DIALOG, AllTrayPromptDialog))
#define ALL_TRAY_PROMPT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ALL_TRAY_TYPE_PROMPT_DIALOG, AllTrayPromptDialogClass))
#define ALL_TRAY_IS_PROMPT_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ALL_TRAY_TYPE_PROMPT_DIALOG))
#define ALL_TRAY_IS_PROMPT_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ALL_TRAY_TYPE_PROMPT_DIALOG))
#define ALL_TRAY_PROMPT_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ALL_TRAY_TYPE_PROMPT_DIALOG, AllTrayPromptDialogClass))

typedef struct _AllTrayPromptDialog AllTrayPromptDialog;
typedef struct _AllTrayPromptDialogClass AllTrayPromptDialogClass;
typedef struct _AllTrayPromptDialogPrivate AllTrayPromptDialogPrivate;

struct _AllTrayPromptDialog {
	GtkDialog parent_instance;
	AllTrayPromptDialogPrivate * priv;
};

struct _AllTrayPromptDialogClass {
	GtkDialogClass parent_class;
};

struct _AllTrayPromptDialogPrivate {
	GtkLabel* _lblPrompt;
};


static gpointer all_tray_prompt_dialog_parent_class = NULL;

GType all_tray_prompt_dialog_get_type (void);
#define ALL_TRAY_PROMPT_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), ALL_TRAY_TYPE_PROMPT_DIALOG, AllTrayPromptDialogPrivate))
enum  {
	ALL_TRAY_PROMPT_DIALOG_DUMMY_PROPERTY
};
static void all_tray_prompt_dialog_create_widgets (AllTrayPromptDialog* self);
AllTrayPromptDialog* all_tray_prompt_dialog_new (void);
AllTrayPromptDialog* all_tray_prompt_dialog_construct (GType object_type);
static void all_tray_prompt_dialog_finalize (GObject* obj);



AllTrayPromptDialog* all_tray_prompt_dialog_construct (GType object_type) {
	AllTrayPromptDialog * self;
	self = g_object_newv (object_type, 0, NULL);
	gtk_window_set_title ((GtkWindow*) self, _ ("Select an Application"));
	gtk_dialog_set_has_separator ((GtkDialog*) self, FALSE);
	gtk_container_set_border_width ((GtkContainer*) self, (guint) 5);
	all_tray_prompt_dialog_create_widgets (self);
	return self;
}


AllTrayPromptDialog* all_tray_prompt_dialog_new (void) {
	return all_tray_prompt_dialog_construct (ALL_TRAY_TYPE_PROMPT_DIALOG);
}


static void all_tray_prompt_dialog_create_widgets (AllTrayPromptDialog* self) {
	const char* _tmp0_;
	char* labelText;
	GtkLabel* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = NULL;
	labelText = (_tmp0_ = _ ("Click on a window of the software you wish to dock.\n\nNote: do not click on the window border itself. Click inside\nthe window; particularly on Compiz, clicking on the title bar\nor the other sides of the window border DOES NOT WORK."), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	_tmp1_ = NULL;
	self->priv->_lblPrompt = (_tmp1_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (labelText)), (self->priv->_lblPrompt == NULL) ? NULL : (self->priv->_lblPrompt = (g_object_unref (self->priv->_lblPrompt), NULL)), _tmp1_);
	gtk_box_pack_start ((GtkBox*) ((GtkDialog*) self)->vbox, (GtkWidget*) self->priv->_lblPrompt, FALSE, TRUE, (guint) 0);
	gtk_widget_show_all ((GtkWidget*) self);
	labelText = (g_free (labelText), NULL);
}


static void all_tray_prompt_dialog_class_init (AllTrayPromptDialogClass * klass) {
	all_tray_prompt_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (AllTrayPromptDialogPrivate));
	G_OBJECT_CLASS (klass)->finalize = all_tray_prompt_dialog_finalize;
}


static void all_tray_prompt_dialog_instance_init (AllTrayPromptDialog * self) {
	self->priv = ALL_TRAY_PROMPT_DIALOG_GET_PRIVATE (self);
}


static void all_tray_prompt_dialog_finalize (GObject* obj) {
	AllTrayPromptDialog * self;
	self = ALL_TRAY_PROMPT_DIALOG (obj);
	(self->priv->_lblPrompt == NULL) ? NULL : (self->priv->_lblPrompt = (g_object_unref (self->priv->_lblPrompt), NULL));
	G_OBJECT_CLASS (all_tray_prompt_dialog_parent_class)->finalize (obj);
}


GType all_tray_prompt_dialog_get_type (void) {
	static GType all_tray_prompt_dialog_type_id = 0;
	if (all_tray_prompt_dialog_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (AllTrayPromptDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) all_tray_prompt_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (AllTrayPromptDialog), 0, (GInstanceInitFunc) all_tray_prompt_dialog_instance_init, NULL };
		all_tray_prompt_dialog_type_id = g_type_register_static (GTK_TYPE_DIALOG, "AllTrayPromptDialog", &g_define_type_info, 0);
	}
	return all_tray_prompt_dialog_type_id;
}




